join_datasets <- function(data, multimodel, n_members) {
  
  ## Taking the first model as "template"
  output <- data[[1]]
  
  if (is.null(output$fcst$data)) {
    exps <- 'hcst'
  } else {exps <- c('hcst', 'fcst')}
  
  for (exp in exps) {
    
    # New dimensions of the arrays
    if (isFALSE(multimodel)) { ## Individual models
      n_models <- length(data)
      max_members <- max(n_members[[exp]])
    } else if (isTRUE(multimodel)) { ## Multimodel
      n_models <- length(data)
      max_members <- sum(n_members[[exp]])
    } else if (identical(multimodel, 'both')) { ## Individual models and multimodel
      n_models <- length(data) + 1
      max_members <- sum(n_members[[exp]])
    } else {
      stop('Incorrect "Multimodel" in the recipe. It must be "yes", "no" or "both".')
    }
    dims <- dim(data[[1]][[exp]]$data)
    dims['dat'] <- n_models
    dims['ensemble'] <- max_members

    # Creating array for models and/or multimodel
    output[[exp]]$data <- array(data = NA, dim = dims)

    ## Filling the models
    fill_data <- function(data, n_members, dims) {
      # data: a list of hcst/fcst$data from each model 
      # n_members: a vector of members of each model
      # dims: c(dat = n_models, ensemble = max_members)

      .fill_data <- function(data, n_members, dims) {
        output <- rep(NA, length = dims['ensemble'])
        output[1:n_members] <- data
        return(output)
      }

      data_list <- vector('list', length = length(data))
      for (i_model in 1:length(data)) {
      data_list[[i_model]] <- multiApply::Apply(data = data[[i_model]], 
                                  target_dims = 'ensemble', output_dims = 'ensemble',
                                  fun = .fill_data, n_members = n_members[i_model],
                                  dims = dims)$output1
      }
      if (dims['dat'] > length(data)) { # Multimodel = 'both'
        data_list[[as.numeric(dims['dat'])]] <- array(as.numeric(NA), dim = dim(data_list[[1]]))
      }
      output <- array(unlist(data_list), dim = c(dims['ensemble'], dim(data_list[[1]])[-c(1, 2)], dims['dat']))
      return(output)
    }

    data_exp_data <- lapply(data, '[[', c(exp, 'data'))
    output[[exp]]$data <- fill_data(data = (data_exp_data), n_members = n_members[[exp]], 
                                    dims = dims[c('dat', 'ensemble')])
    
    ## Filling the multimodel
    if (!isFALSE(multimodel)) {
        data_multi <- CSTools::MergeDims(data = output[[exp]]$data,
                                         merge_dims = c('ensemble', 'dat'),
                                         rename_dim = 'ensemble',
                                         na.rm = TRUE)

      if (isTRUE(multimodel)) { ## Multimodel
        output[[exp]]$data <- data_multi
        output[[exp]]$data <- s2dv::InsertDim(data = output[[exp]]$data,
                                              posdim = 1,
                                              lendim = 1,
                                              name = 'dat')

      } else if (identical(multimodel, 'both')) { ## Individual models and multimodel
        tmp <- asplit(output[[exp]]$data, which(names(dim(output[[exp]]$data)) == 'dat'))
        tmp[[n_models]] <- data_multi
        output[[exp]]$data <- array(unlist(tmp), dim = c(dim(tmp[[1]]), dat = n_models))
      }
    }

    # Reorder data back to standard
    tmp <- match(c('dat', 'var', 'sday', 'sweek', 'syear', 'time', 'latitude', 'longitude', 'ensemble'),
                 names(dim(output[[exp]]$data)))
    output[[exp]]$data <- aperm(output[[exp]]$data, tmp[!is.na(tmp)])

  }
  
  return(output)
}
